@@ -12,8 +12,6 @@
 #include <libxml++/nodes/commentnode.h>
 #include <libxml++/nodes/cdatanode.h>
 #include <libxml++/nodes/processinginstructionnode.h>
-#include <libxml++/nodes/xincludestart.h>
-#include <libxml++/nodes/xincludeend.h>
 #include <libxml++/exceptions/internal_error.h>
 #include <libxml++/attributedeclaration.h>
 #include <libxml++/attributenode.h>
@@ -24,117 +22,13 @@
 
 #include <iostream>
 
-namespace // anonymous
-{
-// Common part of xmlpp::Node::eval_to_[boolean|number|string]
-xmlXPathObject* eval_common(const Glib::ustring& xpath,
-  const xmlpp::Node::PrefixNsMap* namespaces,
-  xmlpp::XPathResultType* result_type, xmlNode* node)
-{
-  auto ctxt = xmlXPathNewContext(node->doc);
-  if (!ctxt)
-    throw xmlpp::internal_error("Could not create XPath context for " + xpath);
-  ctxt->node = node;
-
-  if (namespaces)
-  {
-    for (xmlpp::Node::PrefixNsMap::const_iterator it = namespaces->begin();
-         it != namespaces->end(); ++it)
-      xmlXPathRegisterNs(ctxt,
-        reinterpret_cast<const xmlChar*>(it->first.c_str()),
-        reinterpret_cast<const xmlChar*>(it->second.c_str()));
-  }
-
-  auto xpath_value = xmlXPathEvalExpression(
-    reinterpret_cast<const xmlChar*>(xpath.c_str()), ctxt);
-
-  xmlXPathFreeContext(ctxt);
-
-  if (!xpath_value)
-  {
-    if (result_type)
-      *result_type = xmlpp::XPATH_RESULT_UNDEFINED;
-
-    throw xmlpp::exception("Invalid XPath: " + xpath);
-  }
-
-  if (result_type)
-  {
-    if (xpath_value->type == XPATH_NODESET ||
-        xpath_value->type == XPATH_BOOLEAN ||
-        xpath_value->type == XPATH_NUMBER ||
-        xpath_value->type == XPATH_STRING)
-      *result_type = static_cast<xmlpp::XPathResultType>(xpath_value->type);
-    else
-      *result_type = xmlpp::XPATH_RESULT_UNDEFINED;
-  }
-
-  return xpath_value;
-}
-
-// Common part of all overloaded xmlpp::Node::eval_to_boolean() methods.
-bool eval_common_to_boolean(const Glib::ustring& xpath,
-  const xmlpp::Node::PrefixNsMap* namespaces,
-  xmlpp::XPathResultType* result_type, xmlNode* node)
-{
-  auto xpath_value = eval_common(xpath, namespaces, result_type, node);
-  const int result = xmlXPathCastToBoolean(xpath_value);
-  xmlXPathFreeObject(xpath_value);
-  return static_cast<bool>(result);
-}
-
-// Common part of all overloaded xmlpp::Node::eval_to_number() methods.
-double eval_common_to_number(const Glib::ustring& xpath,
-  const xmlpp::Node::PrefixNsMap* namespaces,
-  xmlpp::XPathResultType* result_type, xmlNode* node)
-{
-  auto xpath_value = eval_common(xpath, namespaces, result_type, node);
-  const double result = xmlXPathCastToNumber(xpath_value);
-  xmlXPathFreeObject(xpath_value);
-  return result;
-}
-
-// Common part of all overloaded xmlpp::Node::eval_to_string() methods.
-Glib::ustring eval_common_to_string(const Glib::ustring& xpath,
-  const xmlpp::Node::PrefixNsMap* namespaces,
-  xmlpp::XPathResultType* result_type, xmlNode* node)
-{
-  auto xpath_value = eval_common(xpath, namespaces, result_type, node);
-  xmlChar* result = xmlXPathCastToString(xpath_value);
-  xmlXPathFreeObject(xpath_value);
-  if (result)
-  {
-    const Glib::ustring uresult(reinterpret_cast<const char*>(result));
-    xmlFree(result);
-    return uresult;
-  }
-  return Glib::ustring();
-}
-
-// Common part of all add_child*() methods.
-xmlpp::Element* add_child_common(const Glib::ustring& name, xmlNode* child, xmlNode* node)
-{
-  if (!node)
-  {
-    xmlFreeNode(child);
-    throw xmlpp::internal_error("Could not add child element node " + name);
-  }
-  xmlpp::Node::create_wrapper(node);
-  return static_cast<xmlpp::Element*>(node->_private);
-}
-
-} // anonymous namespace
-
 namespace xmlpp
 {
 
 Node::Node(xmlNode* node)
   : impl_(node)
 {
-  if (!impl_)
-    throw internal_error("xmlNode pointer cannot be nullptr");
-
-  impl_->_private = this;
+   impl_->_private = this;
 }
 
 Node::~Node()
@@ -148,7 +42,7 @@ const Element* Node::get_parent() const
 Element* Node::get_parent()
 {
   if(!(cobj()->parent && cobj()->parent->type == XML_ELEMENT_NODE))
-    return nullptr;
+    return 0;
 
   Node::create_wrapper(cobj()->parent);
   return static_cast<Element*>(cobj()->parent->_private);
@@ -162,7 +56,7 @@ const Node* Node::get_next_sibling() const
 Node* Node::get_next_sibling()
 {
   if(!cobj()->next)
-    return nullptr;
+    return 0;
 
   Node::create_wrapper(cobj()->next);
   return static_cast<Node*>(cobj()->next->_private);
@@ -176,47 +70,15 @@ const Node* Node::get_previous_sibling() const
 Node* Node::get_previous_sibling()
 {
   if(!cobj()->prev)
-    return nullptr;
+    return 0;
 
   Node::create_wrapper(cobj()->prev);
   return static_cast<Node*>(cobj()->prev->_private);
 }
 
-static Node* _convert_node(xmlNode* node)
-{
-  Node* res = nullptr;
-  if(node)
-  {
-    Node::create_wrapper(node);
-    res = static_cast<Node*>(node->_private);
-  }
-  return res;
-}
-
-Node* Node::get_first_child(const Glib::ustring& name)
-{
-  auto child = impl_->children;
-  if(!child)
-    return nullptr;
-
-  do
-  {
-    if(name.empty() || name == (const char*)child->name)
-      return _convert_node(child);
-  }
-  while((child = child->next));
-   
-  return nullptr;
-}
-
-const Node* Node::get_first_child(const Glib::ustring& name) const
-{
-  return const_cast<Node*>(this)->get_first_child(name);
-}
-
 Node::NodeList Node::get_children(const Glib::ustring& name)
 {
-   auto child = impl_->children;
+   xmlNode* child = impl_->children;
    if(!child)
      return NodeList();
 
@@ -224,7 +86,10 @@ Node::NodeList Node::get_children(const Glib::ustring& name)
    do
    {
       if(name.empty() || name == (const char*)child->name)
-        children.push_back(_convert_node(child));
+      {
+        Node::create_wrapper(child);
+        children.push_back(reinterpret_cast<Node*>(child->_private));
+      }
    }
    while((child = child->next));
    
@@ -239,80 +104,73 @@ const Node::NodeList Node::get_children(const Glib::ustring& name) const
 Element* Node::add_child(const Glib::ustring& name,
                          const Glib::ustring& ns_prefix)
 {
-  auto child = create_new_child_node(name, ns_prefix);
-  auto node = xmlAddChild(impl_, child);
-  return add_child_common(name, child, node);
+  _xmlNode* child = create_new_child_node(name, ns_prefix);
+  if(!child)
+    return 0;
+
+  _xmlNode* node = xmlAddChild(impl_, child);
+  if(!node)
+    return 0;
+ 
+  Node::create_wrapper(node);
+  return static_cast<Element*>(node->_private);
 }
 
 Element* Node::add_child(xmlpp::Node* previous_sibling, 
                          const Glib::ustring& name,
                          const Glib::ustring& ns_prefix)
 {
-  if (!previous_sibling)
-    return nullptr;
+  if(!previous_sibling)
+    return 0;
 
-  auto child = create_new_child_node(name, ns_prefix);
-  auto node = xmlAddNextSibling(previous_sibling->cobj(), child);
-  return add_child_common(name, child, node);
+  _xmlNode* child = create_new_child_node(name, ns_prefix);
+  if(!child)
+    return 0;
+
+  _xmlNode* node = xmlAddNextSibling(previous_sibling->cobj(), child);
+  if(!node)
+    return 0;
+
+  Node::create_wrapper(node);
+  return static_cast<Element*>(node->_private);
 }
 
 Element* Node::add_child_before(xmlpp::Node* next_sibling, 
                          const Glib::ustring& name,
                          const Glib::ustring& ns_prefix)
 {
-  if (!next_sibling)
-    return nullptr;
+  if(!next_sibling)
+    return 0;
 
-  auto child = create_new_child_node(name, ns_prefix);
-  auto node = xmlAddPrevSibling(next_sibling->cobj(), child);
-  return add_child_common(name, child, node);
-}
+  _xmlNode* child = create_new_child_node(name, ns_prefix);
+  if(!child)
+    return 0;
 
-Element* Node::add_child_with_new_ns(const Glib::ustring& name,
-  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
-{
-  auto child = create_new_child_node_with_new_ns(name, ns_uri, ns_prefix);
-  auto node = xmlAddChild(impl_, child);
-  return add_child_common(name, child, node);
-}
+  _xmlNode* node = xmlAddPrevSibling(next_sibling->cobj(), child);
+  if(!node)
+    return 0;
 
-Element* Node::add_child_with_new_ns(xmlpp::Node* previous_sibling,
-  const Glib::ustring& name,
-  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
-{
-  if (!previous_sibling)
-    return nullptr;
-
-  auto child = create_new_child_node_with_new_ns(name, ns_uri, ns_prefix);
-  auto node = xmlAddNextSibling(previous_sibling->cobj(), child);
-  return add_child_common(name, child, node);
-}
-
-Element* Node::add_child_before_with_new_ns(xmlpp::Node* next_sibling,
-  const Glib::ustring& name,
-  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
-{
-  if (!next_sibling)
-    return nullptr;
-
-  auto child = create_new_child_node_with_new_ns(name, ns_uri, ns_prefix);
-  auto node = xmlAddPrevSibling(next_sibling->cobj(), child);
-  return add_child_common(name, child, node);
+  Node::create_wrapper(node);
+  return static_cast<Element*>(node->_private);
 }
 
 _xmlNode* Node::create_new_child_node(const Glib::ustring& name, const Glib::ustring& ns_prefix)
 {
-   xmlNs* ns = nullptr;
+   xmlNs* ns = 0;
 
    if(impl_->type != XML_ELEMENT_NODE)
    {
-     throw internal_error("You can only add child nodes to element nodes");
+      #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
+      throw internal_error("You can only add child nodes to element nodes");
+      #else
+      return 0;
+      #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
    }
 
    if(ns_prefix.empty())
    {
      //Retrieve default namespace if it exists
-     ns = xmlSearchNs(impl_->doc, impl_, nullptr);
+     ns = xmlSearchNs(impl_->doc, impl_, 0);
    }
    else
    {
@@ -320,51 +178,24 @@ _xmlNode* Node::create_new_child_node(const Glib::ustring& name, const Glib::ust
      ns = xmlSearchNs(impl_->doc, impl_, (const xmlChar*)ns_prefix.c_str());
      if (!ns)
      {
+       #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
        throw exception("The namespace prefix (" + ns_prefix + ") has not been declared.");
+       #else
+       return 0;
+       #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
      }
    }
 
    return xmlNewNode(ns, (const xmlChar*)name.c_str());
 }
 
-_xmlNode* Node::create_new_child_node_with_new_ns(const Glib::ustring& name,
-  const Glib::ustring& ns_uri, const Glib::ustring& ns_prefix)
-{
-  if (impl_->type != XML_ELEMENT_NODE)
-    throw internal_error("You can only add child nodes to element nodes.");
-
-  auto child = xmlNewNode(nullptr, (const xmlChar*)name.c_str());
-  if (!child)
-    throw internal_error("Could not create new element node.");
-
-  auto ns = xmlNewNs(child, (const xmlChar*)(ns_uri.empty() ? nullptr : ns_uri.c_str()),
-                       (const xmlChar*)(ns_prefix.empty() ? nullptr : ns_prefix.c_str()) );
-  // xmlNewNs() does not create a namespace node for the predefined xml prefix.
-  // It's usually defined in the document and not in any specific node.
-  if (!ns && ns_prefix == "xml")
-  {
-    ns = xmlSearchNs(impl_->doc, impl_, (const xmlChar*)ns_prefix.c_str());
-    if (ns && (ns_uri != (ns->href ? (const char*)ns->href : "")))
-      ns = nullptr;
-  }
-  if (!ns)
-  {
-    xmlFreeNode(child);
-    throw internal_error("Could not create new namespace node.");
-  }
-
-  xmlSetNs(child, ns);
-
-  return child;
-}
 
 void Node::remove_child(Node* node)
 {
   //TODO: Allow a node to be removed without deleting it, to allow it to be moved?
   //This would require a more complex memory management API.
-  if (!node)
-    return;
-  auto cnode = node->cobj();
+  
+  xmlNode* cnode = node->cobj();
   Node::free_wrappers(cnode); //This delete the C++ node (not this) itself.
   xmlUnlinkNode(cnode);
   xmlFreeNode(cnode);
@@ -372,45 +203,33 @@ void Node::remove_child(Node* node)
 
 Node* Node::import_node(const Node* node, bool recursive)
 {
-  if (!node)
-    return nullptr;
-
   //Create the node, by copying:
-  auto imported_node = xmlDocCopyNode(const_cast<xmlNode*>(node->cobj()), impl_->doc, recursive);
+  xmlNode* imported_node = xmlDocCopyNode(const_cast<xmlNode*>(node->cobj()), impl_->doc, recursive);
   if (!imported_node)
   {
-    throw exception("Unable to copy the node that shall be imported");
-  }
-
-  if (imported_node->type == XML_ATTRIBUTE_NODE && impl_->type == XML_ELEMENT_NODE)
-  {
-    auto old_attr = xmlHasNsProp(impl_, imported_node->name,
-      imported_node->ns ? imported_node->ns->href : nullptr);
-    if (old_attr && old_attr->type != XML_ATTRIBUTE_DECL)
-    {
-      // *this has an attribute with the same name as the imported attribute.
-      // xmlAddChild() will delete the existing attribute.
-      // Delete the C++ wrapper before the call to xmlAddChild().
-      Node::free_wrappers(reinterpret_cast<xmlNode*>(old_attr));
-    }
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
+    throw exception("Unable to import node");
+    #else
+    return 0;
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
   }
 
   //Add the node:
-  auto added_node = xmlAddChild(this->cobj(), imported_node);
-
+  xmlNode* added_node = xmlAddChild(this->cobj(),imported_node);
   if (!added_node)
   {
     Node::free_wrappers(imported_node);
     xmlFreeNode(imported_node);
 
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
     throw exception("Unable to add imported node to current node");
+    #else
+    return 0;
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
   }
 
-  // Usually added_node == imported_node, but a text node is merged with an
-  // adjacent text node. In that case, xmlAddChild() frees imported_node, and
-  // added_node is a pointer to the old text node.
-  Node::create_wrapper(added_node);
-  return static_cast<Node*>(added_node->_private);
+  Node::create_wrapper(imported_node);
+  return static_cast<Node*>(imported_node->_private);
 }
 
 Glib::ustring Node::get_name() const
@@ -449,13 +268,17 @@ Glib::ustring Node::get_path() const
 
 static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
 {
-  auto result = xmlXPathEval((const xmlChar*)xpath.c_str(), ctxt);
+  xmlXPathObject* result = xmlXPathEval((const xmlChar*)xpath.c_str(), ctxt);
 
   if(!result)
   {
     xmlXPathFreeContext(ctxt);
 
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
     throw exception("Invalid XPath: " + xpath);
+    #else
+    return NodeSet();
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
   }
 
   if(result->type != XPATH_NODESET)
@@ -463,10 +286,14 @@ static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
     xmlXPathFreeObject(result);
     xmlXPathFreeContext(ctxt);
 
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
     throw internal_error("Only nodeset result types are supported.");
+    #else
+    return NodeSet();
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
   }
 
-  auto nodeset = result->nodesetval;
+  xmlNodeSet* nodeset = result->nodesetval;
   NodeSet nodes;
   if( nodeset && !xmlXPathNodeSetIsEmpty(nodeset))
   {
@@ -474,13 +301,7 @@ static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
     nodes.reserve(count);
     for (int i = 0; i != count; ++i)
     {
-      auto cnode = xmlXPathNodeSetItem(nodeset, i);
-      if(!cnode)
-      {
-        std::cerr << "Node::find_impl: The xmlNode was null." << std::endl;
-        continue;
-      }
-
+      xmlNode* cnode = xmlXPathNodeSetItem(nodeset, i);
       if(cnode->type == XML_NAMESPACE_DECL)
       {
         //In this case we would cast it to a xmlNs*,
@@ -492,7 +313,7 @@ static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
       //TODO: Check for other cnode->type values?
   
       Node::create_wrapper(cnode);
-      auto cppNode = static_cast<Node*>(cnode->_private);
+      Node* cppNode = static_cast<Node*>(cnode->_private);
       nodes.push_back(cppNode);
     }
   }
@@ -509,9 +330,7 @@ static NodeSet find_impl(xmlXPathContext* ctxt, const Glib::ustring& xpath)
 
 NodeSet Node::find(const Glib::ustring& xpath) const
 {
-  auto ctxt = xmlXPathNewContext(impl_->doc);
-  if (!ctxt)
-    throw internal_error("Could not create XPath context for " + xpath);
+  xmlXPathContext* ctxt = xmlXPathNewContext(impl_->doc);
   ctxt->node = impl_;
   
   return find_impl(ctxt, xpath);
@@ -520,9 +339,7 @@ NodeSet Node::find(const Glib::ustring& xpath) const
 NodeSet Node::find(const Glib::ustring& xpath,
 		   const PrefixNsMap& namespaces) const
 {
-  auto ctxt = xmlXPathNewContext(impl_->doc);
-  if (!ctxt)
-    throw internal_error("Could not create XPath context for " + xpath);
+  xmlXPathContext* ctxt = xmlXPathNewContext(impl_->doc);
   ctxt->node = impl_;
 
   for (PrefixNsMap::const_iterator it=namespaces.begin();
@@ -534,44 +351,9 @@ NodeSet Node::find(const Glib::ustring& xpath,
   return find_impl(ctxt, xpath);
 }
 
-bool Node::eval_to_boolean(const Glib::ustring& xpath, XPathResultType* result_type) const
-{
-  return eval_common_to_boolean(xpath, nullptr, result_type, impl_);
-}
-
-bool Node::eval_to_boolean(const Glib::ustring& xpath, const PrefixNsMap& namespaces,
-  XPathResultType* result_type) const
-{
-  return eval_common_to_boolean(xpath, &namespaces, result_type, impl_);
-}
-
-double Node::eval_to_number(const Glib::ustring& xpath, XPathResultType* result_type) const
-{
-  return eval_common_to_number(xpath, nullptr, result_type, impl_);
-}
-
-double Node::eval_to_number(const Glib::ustring& xpath, const PrefixNsMap& namespaces,
-  XPathResultType* result_type) const
-{
-  return eval_common_to_number(xpath, &namespaces, result_type, impl_);
-}
-
-Glib::ustring Node::eval_to_string(const Glib::ustring& xpath, XPathResultType* result_type) const
-{
-  return eval_common_to_string(xpath, nullptr, result_type, impl_);
-}
-
-Glib::ustring Node::eval_to_string(const Glib::ustring& xpath, const PrefixNsMap& namespaces,
-  XPathResultType* result_type) const
-{
-  return eval_common_to_string(xpath, &namespaces, result_type, impl_);
-}
-
 Glib::ustring Node::get_namespace_prefix() const
 {
-  if(impl_->type == XML_DOCUMENT_NODE ||
-     impl_->type == XML_HTML_DOCUMENT_NODE ||
-     impl_->type == XML_ENTITY_DECL)
+  if(impl_->type == XML_DOCUMENT_NODE || impl_->type == XML_ENTITY_DECL)
   {
     //impl_ is actually of type xmlDoc or xmlEntity, instead of just xmlNode.
     //libxml does not always use GObject-style inheritance, so xmlDoc and
@@ -588,7 +370,8 @@ Glib::ustring Node::get_namespace_prefix() const
     const xmlAttribute* const attr = reinterpret_cast<const xmlAttribute*>(impl_);
     return attr->prefix ? (const char*)attr->prefix : "";
   }
-  else if(impl_->ns && impl_->ns->prefix)
+
+  if(impl_ && impl_->ns && impl_->ns->prefix)
     return (char*)impl_->ns->prefix;
   else
     return Glib::ustring();
@@ -597,7 +380,6 @@ Glib::ustring Node::get_namespace_prefix() const
 Glib::ustring Node::get_namespace_uri() const
 {
   if(impl_->type == XML_DOCUMENT_NODE ||
-     impl_->type == XML_HTML_DOCUMENT_NODE ||
      impl_->type == XML_ENTITY_DECL ||
      impl_->type == XML_ATTRIBUTE_DECL)
   {
@@ -611,7 +393,7 @@ Glib::ustring Node::get_namespace_uri() const
     return Glib::ustring();
   }
 
-  if(impl_->ns && impl_->ns->href)
+  if(impl_ && impl_->ns && impl_->ns->href)
     return (char*)impl_->ns->href;
   else
     return Glib::ustring();
@@ -621,11 +403,15 @@ void Node::set_namespace(const Glib::ustring& ns_prefix)
 {
   if (impl_->type == XML_ATTRIBUTE_DECL)
   {
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
     throw exception("Can't set the namespace of an attribute declaration");
+    #else
+    return;
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
   }
 
   //Look for the existing namespace to use:
-  auto ns = xmlSearchNs( cobj()->doc, cobj(), (xmlChar*)(ns_prefix.empty() ? nullptr : ns_prefix.c_str()) );
+  xmlNs* ns = xmlSearchNs( cobj()->doc, cobj(), (xmlChar*)(ns_prefix.empty() ? 0 : ns_prefix.c_str()) );
   if(ns)
   {
       //Use it for this element:
@@ -633,7 +419,9 @@ void Node::set_namespace(const Glib::ustring& ns_prefix)
   }
   else
   {
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
     throw exception("The namespace (" + ns_prefix + ") has not been declared.");
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
   }
 }
 
@@ -641,8 +429,8 @@ void Node::create_wrapper(xmlNode* node)
 {
   if(node->_private)
   {
-    //Node already wrapped, skip
-    return;
+	  //Node already wrapped, skip
+	  return;
   }
 
   switch (node->type)
@@ -703,18 +491,7 @@ void Node::create_wrapper(xmlNode* node)
       node->_private = new xmlpp::EntityReference(node);
       break;
     }
-    case XML_XINCLUDE_START:
-    {
-      node->_private = new xmlpp::XIncludeStart(node);
-      break;
-    }
-    case XML_XINCLUDE_END:
-    {
-      node->_private = new xmlpp::XIncludeEnd(node);
-      break;
-    }
     case XML_DOCUMENT_NODE:
-    case XML_HTML_DOCUMENT_NODE:
     {
       // do nothing. For Documents it's the wrapper that is the owner.
       break;
@@ -743,7 +520,7 @@ void Node::free_wrappers(xmlNode* node)
   if (node->type != XML_ENTITY_REF_NODE)
   {
     //Walk the children list.
-    for (auto child = node->children; child; child = child->next)
+    for (xmlNode* child = node->children; child; child = child->next)
       free_wrappers(child);
   }
 
@@ -753,22 +530,21 @@ void Node::free_wrappers(xmlNode* node)
     //Node types that have no properties
     case XML_DTD_NODE:
       delete static_cast<Dtd*>(node->_private);
-      node->_private = nullptr;
+      node->_private = 0;
       return;
     case XML_ATTRIBUTE_NODE:
     case XML_ELEMENT_DECL:
     case XML_ATTRIBUTE_DECL:
     case XML_ENTITY_DECL:
       delete static_cast<Node*>(node->_private);
-      node->_private = nullptr;
+      node->_private = 0;
       return;
     case XML_DOCUMENT_NODE:
-    case XML_HTML_DOCUMENT_NODE:
       //Do not free now. The Document is usually the one who owns the caller.
       return;
     default:
       delete static_cast<Node*>(node->_private);
-      node->_private = nullptr;
+      node->_private = 0;
       break;
   }
 
@@ -777,7 +553,7 @@ void Node::free_wrappers(xmlNode* node)
   //_xmlNode::properties would be a nonsense value, leading to crashes,
   //(and shown as valgrind warnings), so we return above, to avoid 
   //checking it here.
-  for(auto attr = node->properties; attr; attr = attr->next)
+  for(xmlAttr* attr = node->properties; attr; attr = attr->next)
     free_wrappers(reinterpret_cast<xmlNode*>(attr));
 }
 
