@@ -50,12 +50,17 @@ void DomParser::parse_file(const Glib::ustring& filename)
 
   if(!context_)
   {
-    throw internal_error("Could not create parser context\n" + format_xml_error());
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
+    throw internal_error("Couldn't create parsing context\n" + format_xml_error());
+    #else
+    return;
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
   }
 
   if(context_->directory == 0)
   {
-    context_->directory = xmlParserGetDirectory(filename.c_str());
+    char* directory = xmlParserGetDirectory(filename.c_str());
+    context_->directory = (char*) xmlStrdup((xmlChar*) directory);
   }
 
   parse_context();
@@ -73,7 +78,11 @@ void DomParser::parse_memory_raw(const unsigned char* contents, size_type bytes_
 
   if(!context_)
   {
-    throw internal_error("Could not create parser context\n" + format_xml_error());
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
+    throw internal_error("Couldn't create parsing context\n" + format_xml_error());
+    #else
+    return;
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
   }
 
   parse_context();
@@ -95,29 +104,28 @@ void DomParser::parse_context()
 
   if(!context_)
   {
-    throw internal_error("Parser context not initialized\n" + format_xml_error());
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
+    throw internal_error("Context not initialized\n" + format_xml_error());
+    #else
+    return;
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
   }
 
-  const int parseError = xmlParseDocument(context_);
+  xmlParseDocument(context_);
 
-  try
-  {
-    check_for_exception();
-  }
-  catch (...)
-  {
-    release_underlying(); //Free doc_ and context_
-    throw; // re-throw exception
-  }
+  check_for_exception();
 
-  Glib::ustring error_str = format_xml_parser_error(context_);
-  if (error_str.empty() && parseError == -1)
-    error_str = "xmlParseDocument() failed.";
+  const Glib::ustring error_str = format_xml_parser_error(context_);
 
   if(!error_str.empty())
   {
     release_underlying(); //Free doc_ and context_
+
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
     throw parse_error(error_str);
+    #else
+    return;
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
   }
 
   doc_ = new Document(context_->myDoc);
@@ -128,8 +136,11 @@ void DomParser::parse_context()
   //Free the parse context, but keep the document alive so people can navigate the DOM tree:
   //TODO: Why not keep the context alive too?
   Parser::release_underlying();
+
+  check_for_exception();
 }
 
+
 void DomParser::parse_stream(std::istream& in)
 {
   release_underlying(); //Free any existing document.
@@ -138,21 +149,24 @@ void DomParser::parse_stream(std::istream& in)
   xmlResetLastError();
 
   context_ = xmlCreatePushParserCtxt(
-      0, // Setting those two parameters to 0 force the parser
+      0, // setting thoses two parameters to 0 force the parser
       0, // to create a document while parsing.
-      0, // chunk
-      0, // size
-      0); // no filename for fetching external entities
+      0,
+      0,
+      ""); // here should come the filename. I don't know if it is a problem to let it empty
 
   if(!context_)
   {
-    throw internal_error("Could not create parser context\n" + format_xml_error());
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
+    throw internal_error("Couldn't create parsing context\n" + format_xml_error());
+    #else
+    return;
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
   }
 
   initialize_context();
 
   //TODO: Shouldn't we use a Glib::ustring here, and some alternative to std::getline()?
-  int firstParseError = XML_ERR_OK;
   std::string line;
   while(std::getline(in, line))
   {
@@ -160,37 +174,24 @@ void DomParser::parse_stream(std::istream& in)
     // about layout in certain cases.
     line += '\n';
 
-    const int parseError = xmlParseChunk(context_, line.c_str(),
-      line.size() /* This is a std::string, not a ustring, so this is the number of bytes. */, 0);
-
-    // Save the first error code if any, but read on.
-    // More errors might be reported and then thrown by check_for_exception().
-    if (parseError != XML_ERR_OK && firstParseError == XML_ERR_OK)
-      firstParseError = parseError;
+    xmlParseChunk(context_, line.c_str(), line.size() /* This is a std::string, not a ustring, so this is the number of bytes. */, 0);
   }
 
-  const int parseError = xmlParseChunk(context_, 0, 0, 1 /* last chunk */);
-  if (parseError != XML_ERR_OK && firstParseError == XML_ERR_OK)
-    firstParseError = parseError;
+  xmlParseChunk(context_, 0, 0, 1);
 
-  try
-  {
-    check_for_exception();
-  }
-  catch (...)
-  {
-    release_underlying(); //Free doc_ and context_
-    throw; // re-throw exception
-  }
+  check_for_exception();
 
-  Glib::ustring error_str = format_xml_parser_error(context_);
-  if (error_str.empty() && firstParseError != XML_ERR_OK)
-    error_str = "Error code from xmlParseChunk(): " + Glib::ustring::format(firstParseError);
+  const Glib::ustring error_str = format_xml_parser_error(context_);
 
   if(!error_str.empty())
   {
     release_underlying(); //Free doc_ and context_
+
+    #ifdef LIBXMLCPP_EXCEPTIONS_ENABLED
     throw parse_error(error_str);
+    #else
+    return;
+    #endif //LIBXMLCPP_EXCEPTIONS_ENABLED
   }
 
   doc_ = new Document(context_->myDoc);
@@ -202,6 +203,8 @@ void DomParser::parse_stream(std::istream& in)
   //Free the parse context, but keep the document alive so people can navigate the DOM tree:
   //TODO: Why not keep the context alive too?
   Parser::release_underlying();
+
+  check_for_exception();
 }
 
 void DomParser::release_underlying()
@@ -231,3 +234,5 @@ const Document* DomParser::get_document() const
 }
 
 } // namespace xmlpp
+
+
